/*
 *  TOPPERS/ASP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Advanced Standard Profile Kernel
 * 
 *  Copyright (C) 2008-2011 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2015-2016 by 3rd Designing Center
 *              Imageing System Development Division RICOH COMPANY, LTD.
 *  Copyright (C) 2017-2019 by TOPPERS PROJECT Educational Working Group.
 * 
 *  嵭Ԥϡʲ(1)(4)ξ˸¤ꡤܥեȥ
 *  ܥեȥѤΤޤࡥʲƱˤѡʣ
 *  ѡۡʰʲѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *      ޤܥեȥΥ桼ޤϥɥ桼Τʤ
 *      ͳ˴Ťᤫ⡤嵭ԤTOPPERSץȤ
 *      դ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơλŪ
 *  ФŬޤơʤݾڤԤʤޤܥեȥ
 *  ѤˤľŪޤϴŪʤ»˴ؤƤ⡤
 *  Ǥʤ
 * 
 *  @(#) $Id: chip_serial.c 698 2019-01-21 15:51:54Z roi $
 */

/*
 *  ꥢI/OǥХSIO˥ɥ饤
 */

#include <kernel.h>
#include <t_syslog.h>
#include "target_stddef.h"
#include "target_serial.h"
#include "target_syssvc.h"

int  pico_uart_getready( int id );
int  pico_uart_putready( int id );
void  pico_uart_init( int id, int bps );
void  pico_uart_irq_init( int id, int bps );
void  pico_uart_deinit( int id );
void  pico_uart_putc_raw( int id, char c );
char  pico_uart_getc( int id );
void  pico_uart_irq_ctrl_tx( int id, int flag );
void  pico_uart_irq_ctrl_rx( int id, int flag );


/*
 *  ꥢI/OݡȽ֥å
 */
typedef struct sio_port_initialization_block {
	uint32_t base;
	INTNO    intno_usart;
} SIOPINIB;

/*
 *  GPIOݡȽ֥å
 */
typedef struct gpio_port_initialization_block {
	uint32_t  txpin;
	uint32_t  rxpin;
} GPIOINIB;

/*
 *  ꥢI/Oݡȴ֥å
 */
struct sio_port_control_block {
	const SIOPINIB  *p_siopinib;  /* ꥢI/OݡȽ֥å */
	const GPIOINIB  *p_gpioinib;  /* GPIOݡȽ֥å */
	intptr_t        exinf;        /* ĥ */
	bool_t          opnflg;       /* ץѤߥե饰 */
};

/*
 * ꥢI/OݡȽ֥å
 */
const SIOPINIB siopinib_table[TNUM_SIOP] = {
	{(uint32_t)USART1_BASE, (INTNO)INTNO_SIO1},
#if TNUM_SIOP >= 2
	{(uint32_t)USART2_BASE, (INTNO)INTNO_SIO2},
#endif
};

/*
 * GPIOݡȽ֥å
 */
const GPIOINIB gpioinib_table[TNUM_SIOP] = {
	{(uint32_t)0, (uint32_t)1
#if TNUM_SIOP >= 2
	},
	{(uint32_t)20, (uint32_t)21
#endif
	}
};

/*
 *  ꥢI/Oݡȴ֥åΥꥢ
 */
SIOPCB  siopcb_table[TNUM_SIOP];

/*
 *  ꥢI/OݡID֥åФΥޥ
 */
#define INDEX_SIOP(siopid)	((uint_t)((siopid) - 1))
#define get_siopcb(siopid)	(&(siopcb_table[INDEX_SIOP(siopid)]))


static  int  get_siopcb_to_id( SIOPCB *p_siopcb )
{
    int  id = 1;

    if( p_siopcb != get_siopcb(id) ) {
      id++;
    }

    return id;
}

void put_hex(char a, int val)
{
	int i, j;
	target_fput_log(a);
	target_fput_log(' ');
	for(i = 28 ; i >= 0 ; i-= 4){
		j = (val >> i) & 0xf;;
		if(j > 9)
			j += ('A'-10);
		else
			j += '0';
		target_fput_log(j);
	}
	target_fput_log('\n');
}

/*
 *  SIOɥ饤Фν
 */
void
sio_initialize(intptr_t exinf)
{
	SIOPCB	*p_siopcb;
	uint_t	i;

	/*
	 *  ꥢI/Oݡȴ֥åν
	 */
	for (p_siopcb = siopcb_table, i = 0; i < TNUM_SIOP; p_siopcb++, i++) {
		p_siopcb->p_siopinib = &(siopinib_table[i]);
		p_siopcb->p_gpioinib = &(gpioinib_table[i]);
		p_siopcb->opnflg = false;
	}
}


/*
 *  ꥢI/OݡȤΥץ
 */
SIOPCB *
sio_opn_por(ID siopid, intptr_t exinf)
{
	SIOPCB          *p_siopcb;
	const SIOPINIB  *p_siopinib;
	//const GPIOINIB  *p_gpioinib;
	bool_t   opnflg;
	ER       ercd;

	p_siopcb = get_siopcb(siopid);
	p_siopinib = p_siopcb->p_siopinib;
	//p_gpioinib = p_siopcb->p_gpioinib;

	/*
	 *  ץ󤷤ݡȤ뤫opnflgɤǤ
	 */
	opnflg = p_siopcb->opnflg;

	p_siopcb->exinf = exinf;

	if(p_siopinib->base == 0)	/* no usart port */
		goto sio_opn_exit;

	/*
	 *  ϡɥν
	 */
	pico_uart_irq_init( INDEX_SIOP(siopid), BPS_SETTING );
	p_siopcb->opnflg = true;

	/*
	 *  ꥢI/OߤΥޥ롥
	 */
	if (!opnflg) {
		ercd = ena_int(p_siopinib->intno_usart);
	assert(ercd == E_OK);
	}

	// clear interrupt mask for chip

sio_opn_exit:;
	return(p_siopcb);
}

/*
 *  ꥢI/OݡȤΥ
 */
void
sio_cls_por(SIOPCB *p_siopcb)
{
	//const SIOPINIB  *p_siopinib;
	//const GPIOINIB  *p_gpioinib;

	//p_siopinib = p_siopcb->p_siopinib;
	//p_gpioinib = p_siopcb->p_gpioinib;

	pico_uart_deinit( INDEX_SIOP(get_siopcb_to_id(p_siopcb)) );

	/*
	 *  ꥢI/Oߤޥ롥
	 */
	if ((p_siopcb->opnflg)) {
		dis_int(p_siopcb->p_siopinib->intno_usart);

		// interrupt mask for chip
	}
	p_siopcb->opnflg = false;
}

/*
 *  SIOγߥӥ롼
 */

Inline bool_t
sio_putready(SIOPCB* p_siopcb)
{
	return  (bool_t)pico_uart_putready(INDEX_SIOP(get_siopcb_to_id(p_siopcb)));
}

Inline bool_t
sio_getready(SIOPCB* p_siopcb)
{
	return  (bool_t)pico_uart_getready(INDEX_SIOP(get_siopcb_to_id(p_siopcb)));
}

void
sio_usart_isr(intptr_t exinf)
{
	SIOPCB          *p_siopcb;

	p_siopcb = get_siopcb(exinf);

	if (sio_getready(p_siopcb)) {
		sio_irdy_rcv(p_siopcb->exinf);
	}
	if (sio_putready(p_siopcb)) {
		sio_irdy_snd(p_siopcb->exinf);
	}
}

/*
 *  ꥢI/OݡȤؤʸ
 */
bool_t
sio_snd_chr(SIOPCB *p_siopcb, char c)
{
	if (sio_putready(p_siopcb)) {
		pico_uart_putc_raw( INDEX_SIOP(get_siopcb_to_id(p_siopcb)), (char)c );
		return true;
	}
	return false;
}


/*
 *  ꥢI/OݡȤʸ
 */
int_t
sio_rcv_chr(SIOPCB *p_siopcb)
{
	int_t c = -1;

	if (sio_getready(p_siopcb)) {
		c = (int_t)pico_uart_getc( INDEX_SIOP(get_siopcb_to_id(p_siopcb)) );
	}
	return c;
}

/*
 *  ꥢI/OݡȤΥХåε
 */
void
sio_ena_cbr(SIOPCB *p_siopcb, uint_t cbrtn)
{
	switch (cbrtn) {
	case SIO_RDY_SND:
		pico_uart_irq_ctrl_tx( INDEX_SIOP(get_siopcb_to_id(p_siopcb)), 1 );
		break;
	case SIO_RDY_RCV:
		pico_uart_irq_ctrl_rx( INDEX_SIOP(get_siopcb_to_id(p_siopcb)), 1 );
		break;
	}
}

/*
 *  ꥢI/OݡȤΥХåζػ
 */
void
sio_dis_cbr(SIOPCB *p_siopcb, uint_t cbrtn)
{
	switch (cbrtn) {
	case SIO_RDY_SND:
		pico_uart_irq_ctrl_tx( INDEX_SIOP(get_siopcb_to_id(p_siopcb)), 0 );
		break;
	case SIO_RDY_RCV:
		pico_uart_irq_ctrl_rx( INDEX_SIOP(get_siopcb_to_id(p_siopcb)), 0 );
		break;
	}
}


/*
 *  1ʸϡʥݡ󥰤Ǥνϡ
 */
void sio_pol_snd_chr(int8_t c, ID siopid)
{
	pico_uart_putc_raw( INDEX_SIOP(siopid), (char)c );
}


void chip_uart_init(ID siopid)
{
	pico_uart_init((int)INDEX_SIOP(siopid), BPS_SETTING );
}

